@@ -16,7 +16,6 @@ package plugin
 
 import (
 	"context"
-	"crypto/tls"
 	"sync"
 	"time"
 
@@ -33,7 +32,6 @@ import (
 )
 
 const (
-	// connType
 	disable    = "not_tls"
 	require    = "required"
 	verifyCa   = "verify_ca"
@@ -230,7 +228,7 @@ func NewConnManager(
 
 // GetConnection returns an existing connection or creates a new one.
 func (c *ConnManager) GetConnection(
-	connURI uri.URI, //nolint:gocritic
+	connURI *uri.URI,
 	params map[string]string,
 ) (*MongoConn, error) {
 	ck := createConnKey(connURI, params)
@@ -242,7 +240,7 @@ func (c *ConnManager) GetConnection(
 		return conn, nil
 	}
 
-	conn, err := c.create(ck, params)
+	conn, err := c.create(ck)
 	if err != nil {
 		return nil, errs.Wrap(err, "failed to create new connection")
 	}
@@ -252,11 +250,11 @@ func (c *ConnManager) GetConnection(
 
 // getConn returns a connection with given uri if it exists and also updates
 // lastTimeAccess, otherwise returns nil.
-func (c *ConnManager) getConn(ck connKey) *MongoConn { //nolint:gocritic
+func (c *ConnManager) getConn(ck *connKey) *MongoConn {
 	c.connectionsMu.Lock()
 	defer c.connectionsMu.Unlock()
 
-	conn, ok := c.connections[ck]
+	conn, ok := c.connections[*ck]
 	if !ok {
 		return nil
 	}
@@ -267,13 +265,13 @@ func (c *ConnManager) getConn(ck connKey) *MongoConn { //nolint:gocritic
 }
 
 func (c *ConnManager) setConn(
-	ck connKey, //nolint:gocritic
+	ck *connKey,
 	conn *MongoConn,
 ) *MongoConn {
 	c.connectionsMu.Lock()
 	defer c.connectionsMu.Unlock()
 
-	existingConn, ok := c.connections[ck]
+	existingConn, ok := c.connections[*ck]
 	if ok {
 		err := closeSession(context.Background(), conn.session)
 		if err != nil {
@@ -285,7 +283,7 @@ func (c *ConnManager) setConn(
 		return existingConn
 	}
 
-	c.connections[ck] = conn
+	c.connections[*ck] = conn
 
 	return conn
 }
@@ -342,10 +340,9 @@ func (c *ConnManager) housekeeper(ctx context.Context, interval time.Duration) {
 
 // create creates a new connection with given credentials.
 func (c *ConnManager) create(
-	ck connKey, //nolint:gocritic
-	params map[string]string,
+	ck *connKey,
 ) (*MongoConn, error) {
-	opt, err := c.createOptions(ck.uri, params)
+	opt, err := c.createOptions(ck)
 	if err != nil {
 		return nil, err
 	}
@@ -396,32 +393,33 @@ func (c *ConnManager) create(
 
 // get returns a connection with given uri if it exists and also updates lastTimeAccess, otherwise returns nil.
 func (c *ConnManager) createOptions(
-	connURI uri.URI, //nolint:gocritic
-	params map[string]string,
+	ck *connKey,
 ) (*options.ClientOptions, error) {
-	details, err := createTLS(params)
-	if err != nil {
-		return nil, err
-	}
-
 	opt := options.Client()
 
-	if connURI.User() != "" {
+	if ck.uri.User() != "" {
 		creds := options.Credential{}
-		creds.Username = connURI.User()
-		creds.Password = connURI.Password()
+		creds.Username = ck.uri.User()
+		creds.Password = ck.uri.Password()
 		creds.PasswordSet = true
 		opt = opt.SetAuth(creds)
 	}
 
-	if details.TlsConnect != disable {
-		err := c.setTLSConfig(opt, details)
+	if ck.tlsConnect != string(tlsconfig.Disabled) {
+		details, err := createTLS(ck)
 		if err != nil {
 			return nil, err
 		}
+
+		tlsConfig, err := details.GetTLSConfig()
+		if err != nil {
+			return nil, errs.Wrap(err, "failed to get TLS config")
+		}
+
+		opt = opt.SetTLSConfig(tlsConfig)
 	}
 
-	opt.SetHosts([]string{connURI.Addr()})
+	opt.SetHosts([]string{ck.uri.Addr()})
 	opt.SetDirect(true)
 	opt.SetConnectTimeout(c.timeout)
 	opt.SetServerSelectionTimeout(c.timeout)
@@ -430,39 +428,41 @@ func (c *ConnManager) createOptions(
 	return opt, nil
 }
 
-func createTLS(params map[string]string) (*tlsconfig.Details, error) {
+func createTLS(ck *connKey) (*tlsconfig.Details, error) {
 	var (
 		validateCA     = true
 		validateClient = false
-		tlsType        = params[tlsConnectParam]
 	)
 
-	if tlsType == "" {
-		tlsType = disable
+	connectionType, err := tlsconfig.NewTLSConnectionType(ck.tlsConnect)
+	if err != nil {
+		return nil, errs.Wrap(err, "failed to parse TLS connection type")
 	}
 
 	details := tlsconfig.NewDetails(
 		"",
-		tlsType,
-		params[tlsCAParam],
-		params[tlsCertParam],
-		params[tlsKeyParam],
-		params[uriParam],
-		disable,
-		require,
-		verifyCa,
-		verifyFull,
+		ck.rawUri,
+		tlsconfig.WithTLSCaFile(ck.tlsCA),
+		tlsconfig.WithTLSCertFile(ck.tlsCert),
+		tlsconfig.WithTLSKeyFile(ck.tlsKey),
+		tlsconfig.WithTLSConnect(connectionType),
+		tlsconfig.WithAllowedConnections(
+			string(tlsconfig.Disabled),
+			require,
+			verifyCa,
+			verifyFull,
+		),
 	)
 
-	if tlsType == disable || tlsType == require {
+	if ck.tlsConnect == string(tlsconfig.Disabled) || ck.tlsConnect == require {
 		validateCA = false
 	}
 
-	if details.TlsKeyFile != "" || details.TlsCertFile != "" {
+	if details.TLSKeyFile != "" || details.TLSCertFile != "" {
 		validateClient = true
 	}
 
-	err := details.Validate(validateCA, validateClient, validateClient)
+	err = details.Validate(validateCA, validateClient, validateClient)
 	if err != nil {
 		return nil, zbxerr.ErrorInvalidConfiguration.Wrap(err)
 	}
@@ -470,59 +470,6 @@ func createTLS(params map[string]string) (*tlsconfig.Details, error) {
 	return &details, nil
 }
 
-func (c *ConnManager) setTLSConfig(
-	opt *options.ClientOptions,
-	details *tlsconfig.Details,
-) error {
-	var cfg *tls.Config
-	var err error
-
-	switch details.TlsConnect {
-	case "required":
-		cfg, err = c.getRequiredTLSConfig(details)
-		if err != nil {
-			return err
-		}
-	case "verify_ca":
-		cfg, err = details.GetTLSConfig(true)
-		if err != nil {
-			return errs.Wrap(err, "failed to get TLS config for verify_ca connection")
-		}
-
-		cfg.VerifyPeerCertificate = tlsconfig.VerifyPeerCertificateFunc(
-			"",
-			cfg.RootCAs,
-		)
-	case "verify_full":
-		cfg, err = details.GetTLSConfig(false)
-		if err != nil {
-			return errs.Wrap(err, "failed to get TLS config for verify_full connection")
-		}
-	}
-
-	opt.SetTLSConfig(cfg)
-
-	return nil
-}
-
-func (c *ConnManager) getRequiredTLSConfig(
-	details *tlsconfig.Details,
-) (*tls.Config, error) {
-	if details.TlsCaFile != "" {
-		c.log.Warningf(
-			"server CA will not be verified for %s",
-			details.TlsConnect,
-		)
-	}
-
-	clientCerts, err := details.LoadCertificates()
-	if err != nil {
-		return nil, err
-	}
-
-	return &tls.Config{Certificates: clientCerts, InsecureSkipVerify: true}, nil
-}
-
 // updateAccessTime updates the last time a connection was accessed.
 func (conn *MongoConn) updateAccessTime() {
 	conn.lastTimeAccess = time.Now()
@@ -547,14 +494,14 @@ func closeSession(ctx context.Context, session mongo.Session) error {
 	return nil
 }
 
-func createConnKey(uri uri.URI, params map[string]string) connKey {
+func createConnKey(u *uri.URI, params map[string]string) *connKey {
 	tlsType := params[tlsConnectParam]
-	if tlsType == "" {
-		tlsType = disable
+	if tlsType == "" || tlsType == disable {
+		tlsType = string(tlsconfig.Disabled)
 	}
 
-	return connKey{
-		uri:        uri,
+	return &connKey{
+		uri:        *u,
 		rawUri:     params[uriParam],
 		tlsConnect: tlsType,
 		tlsCA:      params[tlsCAParam],
